// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/DressedLeptons.hh"

namespace Rivet {

  class ATLAS_2015_I1351916 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructors
    ATLAS_2015_I1351916(std::string name = "ATLAS_2015_I1351916") 
      : Analysis(name)
    {
      _mode = 0; // pick electron channel by default
      setNeedsCrossSection(true);
    }

    //@}

  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

      const FinalState fs;

      Cut cuts = (Cuts::pT > 25*GeV) && (Cuts::abseta < 4.9);
      if (_mode)  cuts = (Cuts::pT > 20*GeV) && (Cuts::abseta < 2.47); // muon channel version

      IdentifiedFinalState bareleptons(fs);
      bareleptons.acceptIdPair(_mode? PID::MUON : PID::ELECTRON);

      DressedLeptons leptons(fs, bareleptons, 0.1, cuts, true, true);
      addProjection(leptons, "leptons");

      // book dummy histograms for heterogeneous merging
      string label = "NCC";
      string hname = "d01-x01-y01";
      const Scatter2D& ref = refData(hname);
      hname = "d01-x01-y02";
      _h[label + "_pos"] = bookHisto1D(hname, ref);
      hname = "d01-x01-y03";
      _h[label + "_neg"] = bookHisto1D(hname, ref);
      if (!_mode) {
        label = "NCF";
        hname = "d01-x02-y01";
        const Scatter2D& ref_cf = refData(hname);
        hname = "d01-x02-y02";
        _h[label + "_pos"] = bookHisto1D(hname, ref_cf);
        hname = "d01-x02-y03";
        _h[label + "_neg"] = bookHisto1D(hname, ref_cf);
      }

      // book asymmetries
      _s["CC"] = bookScatter2D(1, 1, 1, true);
      if (!_mode)  _s["CF"] = bookScatter2D(1, 2, 1, true);
    }

	  //==========================================================================================


    /// Perform the per-event analysis
    void analyze(const Event& e) {

      
      //---------------------------
      const double weight = e.weight();

      const vector<DressedLepton> &leptons = applyProjection<DressedLeptons>(e, "leptons").dressedLeptons();
      if (leptons.size() != 2)  vetoEvent; // two leptons
      if (leptons[0].threeCharge() * leptons[1].threeCharge() > 0)  vetoEvent; // opposite charge
      
      // assign lepton vs antilepton
      int iPos = int(leptons[1].threeCharge() > 0);
      int iNeg = int(leptons[1].threeCharge() < 0);

      string label = "N";
      if (_mode)  label += "CC"; // only central-central for muons
      else { // electron channel
        double eta1 = leptons[iPos].abseta();
        double eta2 = leptons[iNeg].abseta();
        if ( (eta1 < 2.47 && 2.5 < eta2 && eta2 < 4.9) || (eta2 < 2.47 && 2.5 < eta1 && eta1 < 4.9) ) label += "CF"; // central-forward
        else if (eta1 < 2.47 && eta2 < 2.47)                                                          label += "CC"; // central-central
        else  vetoEvent; // ain't no forward-forward
      }

      double cosThetaStar = CollinsSoperCos(leptons[iNeg].momentum(), leptons[iPos].momentum());
      double mll = (leptons[iPos].momentum() + leptons[iNeg].momentum()).mass();
      label += cosThetaStar < 0.0?  "_neg" : "_pos";
      _h[label]->fill(mll, weight);
      
    } // end of analysis loop


    // Cosine of the decay angle in the Collins-Soper frame
    double CollinsSoperCos(const FourMomentum &l1, const FourMomentum &l2) {
      FourMomentum ll = l1 + l2;
      double nom   = (l1.E() + l1.pz()) * (l2.E() - l2.pz()) - (l1.E() - l1.pz()) * (l2.E() + l2.pz());
      double denom = ll.mass() * sqrt( sqr(ll.mass()) + sqr(ll.pt()) );
      double sign  = ll.pz() > 0.0? 1.0 : -1.0;
      return sign * safediv(nom, denom); // protect against division by zero, you never know...
    }


    /// Normalise histograms etc., after the run
    void finalize() {
      
      const double sf = crossSectionPerEvent();
      for(map<string, Histo1DPtr>::iterator it = _h.begin(); it != _h.end(); ++it) {
        scale(it->second, sf);
      }

      divide(*_h["NCC_pos"] - *_h["NCC_neg"], *_h["NCC_pos"] + *_h["NCC_neg"], _s["CC"]);
      if (!_mode)  divide(*_h["NCF_pos"] - *_h["NCF_neg"], *_h["NCF_pos"] + *_h["NCF_neg"], _s["CF"]);

    }
    //@}
    
    
  protected:
	  
    // Data members like post-cuts event weight counters go here
    size_t _mode;
    
  private:

    map<string, Histo1DPtr> _h;
    map<string, Scatter2DPtr> _s;

  };


  class ATLAS_2015_I1351916_EL : public ATLAS_2015_I1351916 {
  public:
    ATLAS_2015_I1351916_EL() : ATLAS_2015_I1351916("ATLAS_2015_I1351916_EL")
    {
      _mode = 0;
    }
  };

  class ATLAS_2015_I1351916_MU : public ATLAS_2015_I1351916 {
  public:
    ATLAS_2015_I1351916_MU() : ATLAS_2015_I1351916("ATLAS_2015_I1351916_MU")
    {
      _mode = 1;
    }
  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2015_I1351916);
  DECLARE_RIVET_PLUGIN(ATLAS_2015_I1351916_EL);
  DECLARE_RIVET_PLUGIN(ATLAS_2015_I1351916_MU);

} 

